@@ -1,75 +1,34 @@
 # -*- coding: utf-8 -*-
 from __future__ import absolute_import, print_function, with_statement
+from typing import Iterable
+from typing import Union
 
+import logging
 import os
 import re
 import subprocess
 import sys
 import tempfile
 import textwrap
-import warnings
+import glob
 
+from .handler import _check_log_handler
 from .pandoc_download import DEFAULT_TARGET_FOLDER, download_pandoc
 from .py3compat import cast_bytes, cast_unicode, string_types, url2path, urlparse
 
 __author__ = u'Juho Vepsäläinen'
-__version__ = '1.7.0'
+__version__ = '1.8'
 __license__ = 'MIT'
-__all__ = ['convert', 'convert_file', 'convert_text',
+__all__ = ['convert_file', 'convert_text',
            'get_pandoc_formats', 'get_pandoc_version', 'get_pandoc_path',
            'download_pandoc']
 
+# Set up the module level logger
+logger = logging.getLogger(__name__)
 
-def convert(source, to, format=None, extra_args=(), encoding='utf-8',
-            outputfile=None, filters=None):
-    """Converts given `source` from `format` to `to` (deprecated).
-
-    :param str source: Unicode string or bytes or a file path (see encoding)
-
-    :param str to: format into which the input should be converted; can be one of
-            `pypandoc.get_pandoc_formats()[1]`
-
-    :param str format: the format of the inputs; will be inferred if input is a file with an
-            known filename extension; can be one of `pypandoc.get_pandoc_formats()[1]`
-            (Default value = None)
-
-    :param list extra_args: extra arguments (list of strings) to be passed to pandoc
-            (Default value = ())
-
-    :param str encoding: the encoding of the file or the input bytes (Default value = 'utf-8')
-
-    :param str outputfile: output will be written to outfilename or the converted content
-            returned if None (Default value = None)
-
-    :param list filters: pandoc filters e.g. filters=['pandoc-citeproc']
-
-    :returns: converted string (unicode) or an empty string if an outputfile was given
-    :rtype: unicode
-
-    :raises RuntimeError: if any of the inputs are not valid of if pandoc fails with an error
-    :raises OSError: if pandoc is not found; make sure it has been installed and is available at
-            path.
-    """
-    msg = ("Due to possible ambiguity, 'convert()' is deprecated and will be removed in pypandoc 1.8. "
-           "Use 'convert_file()'  or 'convert_text()'.")
-    warnings.warn(msg, DeprecationWarning, stacklevel=2)
-
-    path = _identify_path(source)
-    if path:
-        format = _identify_format_from_path(source, format)
-        input_type = 'path'
-    else:
-        source = _as_unicode(source, encoding)
-        input_type = 'string'
-        if not format:
-            raise RuntimeError("Format missing, but need one (identified source as text as no "
-                               "file with that name was found).")
-    return _convert_input(source, format, input_type, to, extra_args=extra_args,
-                          outputfile=outputfile, filters=filters)
-
-
-def convert_text(source, to, format, extra_args=(), encoding='utf-8',
-                 outputfile=None, filters=None, verify_format=True, sandbox=True):
+def convert_text(source:str, to:str, format:str, extra_args:Iterable=(), encoding:str='utf-8',
+                 outputfile:Union[None, str]=None, filters:Union[Iterable, None]=None, verify_format:bool=True,
+                 sandbox:bool=True, cworkdir:Union[str, None]=None) -> str:
     """Converts given `source` from `format` to `to`.
 
     :param str source: Unicode string or bytes (see encoding)
@@ -105,14 +64,16 @@ def convert_text(source, to, format, extra_args=(), encoding='utf-8',
     source = _as_unicode(source, encoding)
     return _convert_input(source, format, 'string', to, extra_args=extra_args,
                           outputfile=outputfile, filters=filters,
-                          verify_format=verify_format, sandbox=sandbox)
+                          verify_format=verify_format, sandbox=sandbox,
+                          cworkdir=cworkdir)
 
 
-def convert_file(source_file, to, format=None, extra_args=(), encoding='utf-8',
-                 outputfile=None, filters=None, verify_format=True, sandbox=True):
+def convert_file(source_file:Union[list, str], to:str, format:Union[str, None]=None, extra_args:Iterable=(), encoding:str='utf-8',
+                 outputfile:Union[None, str]=None, filters:Union[Iterable, None]=None, verify_format:bool=True,
+                 sandbox:bool=True, cworkdir:Union[str, None]=None) -> str:
     """Converts given `source` from `format` to `to`.
 
-    :param str source_file: file path (see encoding)
+    :param (str, list) source_file: Either a full file path, relative file path, a file patterh (like dir/*.md), or a list if file or file patterns.
 
     :param str to: format into which the input should be converted; can be one of
             `pypandoc.get_pandoc_formats()[1]`
@@ -146,17 +107,39 @@ def convert_file(source_file, to, format=None, extra_args=(), encoding='utf-8',
     """
     if not _identify_path(source_file):
         raise RuntimeError("source_file is not a valid path")
-    format = _identify_format_from_path(source_file, format)
-    return _convert_input(source_file, format, 'path', to, extra_args=extra_args,
+    if _is_network_path(source_file): # if the source_file is an url
+        format = _identify_format_from_path(source_file, format)
+        return _convert_input(source_file, format, 'path', to, extra_args=extra_args,
                           outputfile=outputfile, filters=filters,
-                          verify_format=verify_format, sandbox=sandbox)
+                          verify_format=verify_format, sandbox=sandbox,
+                          cworkdir=cworkdir)
+
+    discovered_source_files = []
+    if isinstance(source_file, str):
+        discovered_source_files += glob.glob(source_file)
+    if isinstance(source_file, list): # a list of possibly file or file patterns. Expand all with glob
+        for filepath in source_file:
+            discovered_source_files.extend(glob.glob(filepath))
+    if len(discovered_source_files) == 1: # behavior for a single file or a pattern
+        format = _identify_format_from_path(discovered_source_files[0], format)
+        return _convert_input(discovered_source_files[0], format, 'path', to, extra_args=extra_args,
+                          outputfile=outputfile, filters=filters,
+                          verify_format=verify_format, sandbox=sandbox,
+                          cworkdir=cworkdir)
+    else: # behavior for multiple  files or file patterns
+        format = _identify_format_from_path(discovered_source_files[0], format)
+        return _convert_input(discovered_source_files, format, 'path', to, extra_args=extra_args,
+                          outputfile=outputfile, filters=filters,
+                          verify_format=verify_format, sandbox=sandbox,
+                          cworkdir=cworkdir)
 
 
-def _identify_path(source):
-    # guard against problems
-    if source is None or not isinstance(source, string_types):
-        return False
-
+def _identify_path(source) -> bool:
+    if isinstance(source, list):
+        for single_source in source:
+            if not _identify_path(single_source):
+                return False
+        return True
     is_path = False
     try:
         is_path = os.path.exists(source)
@@ -167,24 +150,51 @@ def _identify_path(source):
         pass
 
     if not is_path:
-        # check if it's an URL
-        result = urlparse(source)
-        if result.scheme in ["http", "https"]:
-            is_path = True
-        elif result.scheme and result.netloc and result.path:
-            # complete uri including one with a network path
-            is_path = True
-        elif result.scheme == "file" and result.path:
-            is_path = os.path.exists(url2path(source))
+        try:
+            is_path = len(glob.glob(source)) >= 1
+        except UnicodeEncodeError:
+            is_path = len(glob.glob(source.encode('utf-8'))) >= 1
+        except:  # noqa
+            # still false
+            pass
+    
+    if not is_path:
+        try:
+            # check if it's an URL
+            result = urlparse(source)
+            if result.scheme in ["http", "https"]:
+                is_path = True
+            elif result.scheme and result.netloc and result.path:
+                # complete uri including one with a network path
+                is_path = True
+            elif result.scheme == "file" and result.path:
+                is_path = os.path.exists(url2path(source))
+        except AttributeError:
+            pass
 
     return is_path
 
+def _is_network_path(source):
+    try:
+        # check if it's an URL
+        result = urlparse(source)
+        if result.scheme in ["http", "https"]:
+            return True
+        elif result.scheme and result.netloc and result.path:
+            # complete uri including one with a network path
+            return True
+        elif result.scheme == "file" and result.path:
+            return os.path.exists(url2path(source))
+    except AttributeError:
+        pass
+    return False
 
-def _identify_format_from_path(sourcefile, format):
+
+def _identify_format_from_path(sourcefile:str, format:str) -> str:
     return format or os.path.splitext(sourcefile)[1].strip('.')
 
 
-def _as_unicode(source, encoding):
+def _as_unicode(source:any, encoding:str) -> any:
     if encoding != 'utf-8':
         # if a source and a different encoding is given, try to decode the the source into a
         # unicode string
@@ -195,7 +205,7 @@ def _as_unicode(source, encoding):
     return source
 
 
-def _identify_input_type(source, format, encoding='utf-8'):
+def _identify_input_type(source:any, format:str, encoding:str='utf-8'):
     path = _identify_path(source)
     if path:
         format = _identify_format_from_path(source, format)
@@ -267,8 +277,11 @@ def _validate_formats(format, to, outputfile):
     return format, to
 
 
-def _convert_input(source, format, input_type, to, extra_args=(), outputfile=None,
-                   filters=None, verify_format=True, sandbox=True):
+def _convert_input(source, format, input_type, to, extra_args=(),
+                   outputfile=None, filters=None, verify_format=True,
+                   sandbox=True, cworkdir=None):
+    
+    _check_log_handler()
     _ensure_pandoc_path()
 
     if verify_format:
@@ -278,7 +291,13 @@ def _convert_input(source, format, input_type, to, extra_args=(), outputfile=Non
         to = normalize_format(to)
 
     string_input = input_type == 'string'
-    input_file = [source] if not string_input else []
+    if not string_input:
+        if isinstance(source, str):
+            input_file = [source]
+        else:
+            input_file = source
+    else:
+        input_file = []
     args = [__pandoc_path, '--from=' + format]
 
     args.append('--to=' + to)
@@ -307,6 +326,11 @@ def _convert_input(source, format, input_type, to, extra_args=(), outputfile=Non
     files_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), "files")
     new_env["PATH"] = new_env.get("PATH", "") + os.pathsep + files_path
     creation_flag = 0x08000000 if sys.platform == "win32" else 0 # set creation flag to not open pandoc in new console on windows
+
+    old_wd = os.getcwd()
+    if cworkdir and old_wd != cworkdir:
+        os.chdir(cworkdir)
+
     p = subprocess.Popen(
         args,
         stdin=subprocess.PIPE if string_input else None,
@@ -315,6 +339,9 @@ def _convert_input(source, format, input_type, to, extra_args=(), outputfile=Non
         env=new_env,
         creationflags=creation_flag)
 
+    if cworkdir is not None:
+        os.chdir(old_wd)
+
     # something else than 'None' indicates that the process already terminated
     if not (p.returncode is None):
         raise RuntimeError(
@@ -322,11 +349,12 @@ def _convert_input(source, format, input_type, to, extra_args=(), outputfile=Non
                                                                            p.stderr.read())
         )
 
-    try:
-        source = cast_bytes(source, encoding='utf-8')
-    except (UnicodeDecodeError, UnicodeEncodeError):
-        # assume that it is already a utf-8 encoded string
-        pass
+    if string_input:
+        try:
+            source = cast_bytes(source, encoding='utf-8')
+        except (UnicodeDecodeError, UnicodeEncodeError):
+            # assume that it is already a utf-8 encoded string
+            pass
     try:
         stdout, stderr = p.communicate(source if string_input else None)
     except OSError:
@@ -351,11 +379,67 @@ def _convert_input(source, format, input_type, to, extra_args=(), outputfile=Non
         raise RuntimeError(
             'Pandoc died with exitcode "%s" during conversion: %s' % (p.returncode, stderr)
         )
+    
+    # if there is output on stderr, process it and send to logger
+    if stderr:
+        for level, msg in _classify_pandoc_logging(stderr):
+            logger.log(level, msg)
 
     # if there is an outputfile, then stdout is likely empty!
     return stdout
 
 
+def _classify_pandoc_logging(raw, default_level="WARNING"):
+    # Process raw and yield the contained logging levels and messages.
+    # Assumes that the messages are formatted like "[LEVEL] message". If the 
+    # first message does not have a level or any other message has a level 
+    # that does not conform to the pandoc standard, use the default_level 
+    # value instead.
+    
+    # Available pandoc logging levels adapted from:
+    # https://github.com/jgm/pandoc/blob/5e1249481b2e3fc27e845245a0c96c3687a23c3d/src/Text/Pandoc/Logging.hs#L44
+    def get_python_level(pandoc_level):
+        
+        level_map = {"ERROR": 40,
+                     "WARNING": 30,
+                     "INFO": 20,
+                     "DEBUG": 10}
+        
+        if pandoc_level not in level_map:
+            level = level_map[default_level]
+        else:
+            level = level_map[pandoc_level]
+        
+        return level
+    
+    msgs = raw.split("\n")
+    first = msgs.pop(0)
+    
+    search = re.search(r"\[(.*?)\]", first)
+    
+    # Use the default if the first message doesn't have a level
+    if search is None:
+        pandoc_level = default_level
+    else:
+        pandoc_level = first[search.start(1):search.end(1)]
+    
+    log_msgs = [first.replace('[{}] '.format(pandoc_level), '')]
+    
+    for msg in msgs:
+        
+        search = re.search(r"\[(.*?)\]", msg)
+        
+        if search is not None:
+            yield get_python_level(pandoc_level), "\n".join(log_msgs)
+            pandoc_level = msg[search.start(1):search.end(1)]
+            log_msgs = [msg.replace('[{}] '.format(pandoc_level), '')]
+            continue
+        
+        log_msgs.append(msg)
+    
+    yield get_python_level(pandoc_level), "\n".join(log_msgs)
+
+
 def _get_base_format(format):
     '''
     According to http://johnmacfarlane.net/pandoc/README.html#general-options,
@@ -366,7 +450,7 @@ def _get_base_format(format):
     return re.split(r'\+|-', format)[0]
 
 
-def get_pandoc_formats():
+def get_pandoc_formats() -> Iterable:
     '''
     Dynamic preprocessor for Pandoc formats.
     Return 2 lists. "from_formats" and "to_formats".
@@ -399,7 +483,7 @@ def get_pandoc_formats():
     return [f.strip() for f in in_], [f.strip() for f in out]
 
 
-def get_pandoc_formats_pre_1_18():
+def get_pandoc_formats_pre_1_18() -> Iterable:
     '''
     Dynamic preprocessor for Pandoc formats for version < 1.18.
     Return 2 lists. "from_formats" and "to_formats".
@@ -428,7 +512,7 @@ def get_pandoc_formats_pre_1_18():
 
 # copied and adapted from jupyter_nbconvert/utils/pandoc.py, Modified BSD License
 
-def _get_pandoc_version(pandoc_path):
+def _get_pandoc_version(pandoc_path:str) -> str:
     new_env = os.environ.copy()
     creation_flag = 0x08000000 if sys.platform == "win32" else 0 # set creation flag to not open pandoc in new console on windows
     if 'HOME' not in os.environ:
@@ -453,7 +537,7 @@ def _get_pandoc_version(pandoc_path):
     return version
 
 
-def get_pandoc_version():
+def get_pandoc_version() -> str:
     """Gets the Pandoc version if Pandoc is installed.
 
     It will probe Pandoc for its version, cache it and return that value. If a cached version is
@@ -471,7 +555,7 @@ def get_pandoc_version():
     return __version
 
 
-def get_pandoc_path():
+def get_pandoc_path() -> str:
     """Gets the Pandoc path if Pandoc is installed.
 
     It will return a path to pandoc which is used by pypandoc.
@@ -491,12 +575,12 @@ def get_pandoc_path():
     _ensure_pandoc_path()
     return __pandoc_path
 
-def ensure_pandoc_minimal_version(major, minor=0):
+def ensure_pandoc_minimal_version(major:int, minor:int=0) -> bool:
     """Check if the used pandoc fulfill a minimal version requirement.
 
-    :param bool major: pandoc major version, such as 1 or 2.
+    :param int major: pandoc major version, such as 1 or 2.
 
-    :param bool minor: pandoc minor version, such as 10 or 11.
+    :param int minor: pandoc minor version, such as 10 or 11.
 
     :returns: True if the installed pandoc is above the minimal version, False otherwise.
     :rtype: bool
@@ -508,12 +592,12 @@ def ensure_pandoc_minimal_version(major, minor=0):
     
 
 
-def ensure_pandoc_maximal_version(major, minor=9999):
+def ensure_pandoc_maximal_version(major:int, minor:int=9999) -> bool:
     """Check if the used pandoc fulfill a maximal version requirement.
 
-    :param bool major: pandoc major version, such as 1 or 2.
+    :param int major: pandoc major version, such as 1 or 2.
 
-    :param bool minor: pandoc minor version, such as 10 or 11.
+    :param int minor: pandoc minor version, such as 10 or 11.
 
     :returns: True if the installed pandoc is below the maximal version, False otherwise.
     :rtype: bool
@@ -522,10 +606,13 @@ def ensure_pandoc_maximal_version(major, minor=9999):
     if version[0] < int(major): # if we have pandoc1 but major is request to be 2
         return True
     return version[0] <= int(major) and version[1] <= int(minor)
-    
-def _ensure_pandoc_path(quiet=False):
-    global __pandoc_path
 
+
+def _ensure_pandoc_path() -> None:
+    global __pandoc_path
+    
+    _check_log_handler()
+    
     if __pandoc_path is None:
         included_pandoc = os.path.join(os.path.dirname(os.path.realpath(__file__)),
                                        "files", "pandoc")
@@ -567,13 +654,13 @@ def _ensure_pandoc_path(quiet=False):
             # print("Trying: %s" % path)
             try:
                 version_string = _get_pandoc_version(path)
-            except Exception as e:
+            except Exception:
                 # we can't use that path...
                 if os.path.exists(path):
                     # path exist but is not useable -> not executable?
-                    if not quiet:
-                        print("Found %s, but not using it because of an error:" % (path), file=sys.stderr)
-                        print(e, file=sys.stderr)
+                    log_msg = ("Found {}, but not using it because of an "
+                               "error:".format(path))
+                    logging.exception(log_msg)
                 continue
             version = [int(x) for x in version_string.split(".")]
             while len(version) < len(curr_version):
@@ -587,58 +674,63 @@ def _ensure_pandoc_path(quiet=False):
 
         if __pandoc_path is None:
             # Only print hints if requested
-            if not quiet:
-                if os.path.exists('/usr/local/bin/brew'):
-                    sys.stderr.write(textwrap.dedent("""\
-                        Maybe try:
+            if os.path.exists('/usr/local/bin/brew'):
+                logger.info(textwrap.dedent("""\
+                    Maybe try:
 
-                            brew install pandoc
-                    """))
-                elif os.path.exists('/usr/bin/apt-get'):
-                    sys.stderr.write(textwrap.dedent("""\
-                        Maybe try:
-
-                            sudo apt-get install pandoc
-                    """))
-                elif os.path.exists('/usr/bin/yum'):
-                    sys.stderr.write(textwrap.dedent("""\
-                        Maybe try:
-
-                        sudo yum install pandoc
-                    """))
-                sys.stderr.write(textwrap.dedent("""\
-                    See http://johnmacfarlane.net/pandoc/installing.html
-                    for installation options
+                        brew install pandoc
                 """))
-                sys.stderr.write(textwrap.dedent("""\
-                    ---------------------------------------------------------------
+            elif os.path.exists('/usr/bin/apt-get'):
+                logger.info(textwrap.dedent("""\
+                    Maybe try:
 
+                        sudo apt-get install pandoc
                 """))
+            elif os.path.exists('/usr/bin/yum'):
+                logger.info(textwrap.dedent("""\
+                    Maybe try:
+
+                    sudo yum install pandoc
+                """))
+            logger.info(textwrap.dedent("""\
+                See http://johnmacfarlane.net/pandoc/installing.html
+                for installation options
+            """))
+            logger.info(textwrap.dedent("""\
+                ---------------------------------------------------------------
+
+            """))
             raise OSError("No pandoc was found: either install pandoc and add it\n"
                           "to your PATH or or call pypandoc.download_pandoc(...) or\n"
                           "install pypandoc wheels with included pandoc.")
 
 
-def ensure_pandoc_installed(url=None, targetfolder=None, version="latest", quiet=False, delete_installer=False):
+def ensure_pandoc_installed(url:Union[str, None]=None, 
+                            targetfolder:Union[str, None]=None,
+                            version:str="latest",
+                            delete_installer:bool=False) -> None:
     """Try to install pandoc if it isn't installed.
 
     Parameters are passed to download_pandoc()
 
     :raises OSError: if pandoc cannot be installed
     """
+
     # Append targetfolder to the PATH environment variable so it is found by subprocesses
     if targetfolder is not None:
         os.environ["PATH"] = os.environ.get("PATH", "") + os.pathsep + os.path.abspath(os.path.expanduser(targetfolder))
 
     try:
-        # Perform the test quietly if asked
-        _ensure_pandoc_path(quiet=quiet)
+        _ensure_pandoc_path()
 
     except OSError:
-        download_pandoc(url=url, targetfolder=targetfolder, version=version, quiet=quiet, delete_installer=delete_installer)
+        download_pandoc(url=url,
+                        targetfolder=targetfolder,
+                        version=version,
+                        delete_installer=delete_installer)
 
         # Show errors in case of secondary failure
-        _ensure_pandoc_path(quiet=False)
+        _ensure_pandoc_path()
 
 
 # -----------------------------------------------------------------------------
